(function ($) {
  'use strict';

  var defaults = {
    debounce: 100,
    scrollOffset: 10
  };

  function debounce (fn, wait, immediate) {
    var timeout;

    return function() {
      var ctx, args, later, callNow, timeout;

      ctx = this;
      args = arguments;

      later = function() {
        timeout = null;
        if (!immediate) fn.apply(ctx, args);
      };

      callNow = immediate && !timeout;
      clearTimeout(timeout);

      timeout = setTimeout(later, wait);
      if (callNow) {
        fn.apply(ctx, args);
      }
    };
  }

  function Topbar (element, options) {
    this.$el = $(element);
    this.opt = $.extend(true, {}, defaults, options);
    this.lastScroll = 0;

    this.init(this);
  }

  Topbar.prototype = {
    init: function (self) {
      $(window).on('scroll', debounce(function () {
        self.toggleFixed(self);
      }, self.opt.debounce));
    },

    toggleFixed: function (self) {
      var $fixed, offset, scrollTop;

      $fixed = $(self.$el.data('topbar-fixed'));
      offset = self.$el.outerHeight();
      scrollTop = $(window).scrollTop();

      if (Math.abs(self.lastScroll - scrollTop) <= self.opt.scrollOffset) {
        return;
      }

      if (self.lastScroll > scrollTop && offset < scrollTop) {
        $fixed.addClass('show');
      } else {
        $fixed.removeClass('show');
      }

      self.lastScroll = scrollTop;
    }

  };

  $.fn.topbar = function (options) {
    return this.each(function() {
      if (!$.data(this, 'topbar')) {
        $.data(this, 'topbar', new Topbar(this, options));
      }
    });
  };
})(window.jQuery);
